-------------------------------------------------------------------
-- Date 		: 29 Dec 2011
-- Databse		: DB2
-- Description 		: ISAMESSO Audit Log Housekeeping Maintenance Script
-- IMS Version		: 8.1.x / 8.2.x
--
--  Syntax : sp_pruneImsLogOlderThan('<del mode>', <days>)
--  * <del mode> - Mandatory, text, case sensitive 
--				 - 'ABOUT' display script information
--				 - 'DELETE' to perform actual delete 
--				 - 'HELP' display script syntax and example
--				 - 'SIMULATE' to simulate deletion without actual delete
--  * <days> - Mandatory, numeric 
--           - Enter 0 to delete all
--
-- Example : 
--  * call sp_pruneImsLogOlderThan('SIMULATE',30) - Simulate deletion of audit log older than 30 days without performing actual delete
--  * call sp_pruneImsLogOlderThan('DELETE',14)   - Delete audit log older than 2 weeks (14 days) from database
--  * call sp_pruneImsLogOlderThan('DELETE',0)    - Delete all audit log from database
--
-------------------------------------------------------------------
-- 
-- DROP PROCEDURE sp_pruneImsLogOlderThan;
--
-- call sp_pruneImsLogOlderThan('ABOUT',0);
-- call sp_pruneImsLogOlderThan('HELP',0);

-- Tables affected : 
--  IMSLOGUserActivity 
--  IMSLOGUserAdminActivity 
--	IMSLOGUserService
--	IMSLOGSystemMgmtActivity
--	IMSLOGSystemOps(if exist)
--
-- Changes Log :
-- v1.0.1 - v9.5 compatible fix - Cast all integer value with TRIM() and CHAR() for String concatenation 
--        - v9.5 compatible fix - Cast date() and time() with CHAR() for String concatenation 
--        - v9.5 compatible fix - Change CURRENT DATE to CURRENT TIMESTAMP for timestamp variable assignment
--        - v9.5 compatible fix - Replace LEFT() with SUBSTR() since LEFT() is not available in v9.5
--        - Remove unnecessary SPECIFIC from stored procedure declaration 
--        - Make GET DIAGNOSTICS EXCEPTION as the first line in the declaration in order to get correct exception message
-- 		  - Added "COMMIT;" in IMSLogSystemMgmtActivity batch delete loop
--		  - Added database and schema name in the About

CREATE PROCEDURE sp_pruneImsLogOlderThan
(
IN IN_operationCmd VARCHAR(16), 
IN IN_retainDays INTEGER
)
DYNAMIC RESULT SETS 1
MODIFIES SQL DATA
LANGUAGE SQL 
BEGIN
	
	DECLARE @info_script_ver varchar(8) DEFAULT '1.0.1';
	DECLARE @info_start_timestamp TIMESTAMP;
	DECLARE @info_end_timestamp TIMESTAMP;
	DECLARE @info_elapsed_time varchar(32);
	
	DECLARE @var_tsRetainDate TIMESTAMP;
	DECLARE @var_iLogSystemOpsExist INTEGER;
	DECLARE @var_strSQL varchar(2048);
	
	DECLARE @var_del_tmpCnt INTEGER DEFAULT 0;
	DECLARE @var_del_LogUserActivity INTEGER DEFAULT 0;
	DECLARE @var_del_LogUserAdminActivity INTEGER DEFAULT 0;
	DECLARE @var_del_LogUserService INTEGER DEFAULT 0;
	DECLARE @var_del_LogSystemMgmtActivity INTEGER DEFAULT 0;
	DECLARE @var_del_LogSystemOps INTEGER DEFAULT 0;
	DECLARE @var_del_mode varchar(16) DEFAULT '';
	DECLARE @var_backup_schema varchar(32) DEFAULT NULL;
 	DECLARE @var_ims_schema varchar(32) DEFAULT NULL;
 
	DECLARE @val_message varchar(128) DEFAULT NULL;
	DECLARE @exception_msg varchar(128) DEFAULT NULL;
	DECLARE @val_temp_int INTEGER;
	
	DECLARE @cursor_sel CURSOR FOR @var_stmtSQL;
			
	-- result output
	DECLARE @cursor_result_output CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_pruneImsLogOlderThan ('''|| IN_operationCmd || ''',''' || TRIM(CHAR(IN_retainDays)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('  Input  > Delete mode : ' || IN_operationCmd) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  3 id, ('  Input  > Days : ' || TRIM(CHAR(IN_retainDays))) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  4 id, ('  Output > Prune before (YYYYMMDD) : ' || TO_CHAR(@var_tsRetainDate,'YYYYMMDD')) msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT  5 id, ('  Output > Start time : ' || CHAR(date(@info_start_timestamp)) || ' ' || CHAR(time(@info_start_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  6 id, ('  Output > End time : ' || CHAR(date(@info_end_timestamp)) || ' ' || CHAR(time(@info_end_timestamp)) ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  7 id, ('  Output > Total elapsed time : ' || @info_elapsed_time) msg FROM SYSIBM.SYSDUMMY1
		
		union SELECT  8 id, ('  Output > - ' || @var_del_mode ||' IMSLogUserActivity : '|| CHAR(@var_del_LogUserActivity)) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  9 id, ('  Output > - ' || @var_del_mode ||' IMSLogUserAdminActivity : '|| CHAR(@var_del_LogUserAdminActivity)) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 10 id, ('  Output > - ' || @var_del_mode ||' IMSLogUserService : '|| CHAR(@var_del_LogUserService)) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 11 id, ('  Output > - ' || @var_del_mode ||' IMSLogSystemMgmtActivity : '|| CHAR(@var_del_LogSystemMgmtActivity)) msg FROM SYSIBM.SYSDUMMY1
		union SELECT 12 id, ('  Output > - ' || @var_del_mode ||' IMSLogSystemOps : ' || CASE WHEN @var_iLogSystemOpsExist>0 THEN CHAR(@var_del_LogSystemOps) ELSE '(NA)' END) msg FROM SYSIBM.SYSDUMMY1
	) order by id;
	

	DECLARE @cursor_about CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT  1 id, (' ISAMESSO Audit Log Housekeeping Maintenance Script ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('  - Database Name    : ' || CURRENT SERVER )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  3 id, ('  - Schema Name      : ' || CURRENT SCHEMA )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  4 id, ('  - Script Version   : ' || @info_script_ver )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  5 id, ('  - Database Version : ' || (SELECT service_level FROM TABLE (sysproc.env_get_inst_info())) )msg FROM SYSIBM.SYSDUMMY1
	) order by id;

	
	DECLARE @cursor_syntax CURSOR WITH RETURN TO CLIENT FOR
	SELECT msg from
	(
		SELECT 0 id, ('command : sp_pruneImsLogOlderThan ('''|| IN_operationCmd || ''',''' || TRIM(CHAR(IN_retainDays)) || ')' ) msg FROM SYSIBM.SYSDUMMY1
		union SELECT  1 id,  CASE WHEN @val_message IS NULL OR TRIM(@val_message) = '' THEN '' ELSE ' * ' || @val_message END msg FROM SYSIBM.SYSDUMMY1
		union SELECT  2 id, ('Syntax : sp_pruneImsLogOlderThan(''<del mode>'', <days> )' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  6 id, (' * <del mode> - Mandatory, text, case sensitive ' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  7 id, ('  			- ''ABOUT'' display script information' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  8 id, ('  			- ''DELETE'' to perform actual delete' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  9 id, ('  			- ''HELP'' display script syntax and example' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  10 id, ('  			- ''SIMULATE'' to simulate deletion without actual delete' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  11 id, (' * <days> 	- Mandatory, numeric' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  12 id, ('  			- Enter 0 to delete all' )msg FROM SYSIBM.SYSDUMMY1
		union SELECT  20 id, (' ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  21 id, (' Example : ') msg FROM SYSIBM.SYSDUMMY1 
		union SELECT  22 id, (' * sp_pruneImsLogOlderThan(''SIMULATE'',30) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  23 id, ('   - Simulate deletion of audit log older than 30 days without performing actual delete')msg FROM SYSIBM.SYSDUMMY1
		union SELECT  25 id, (' * sp_pruneImsLogOlderThan(''DELETE'',14) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  26 id, ('   - Delete audit log older than 2 weeks (14 days) from database') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  28 id, (' * sp_pruneImsLogOlderThan(''DELETE'',0) ') msg FROM SYSIBM.SYSDUMMY1
		union SELECT  29 id, ('   - Delete all audit log from database') msg FROM SYSIBM.SYSDUMMY1		
		
		
	) order by id;
	
	DECLARE @cursor_select CURSOR FOR @stmt_select;
	
	-- Handle SQLException
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		GET DIAGNOSTICS EXCEPTION 1 @exception_msg = MESSAGE_TEXT;
		
		IF @val_message IS NULL THEN
			SET @val_message = @exception_msg;
		END IF;
		
		IF IN_operationCmd = 'ABOUT' THEN
			IF TRIM(@val_message) = '' THEN
				OPEN @cursor_about;
			ELSE
				OPEN @cursor_syntax;
			END IF;
		ELSEIF IN_operationCmd = 'HELP' THEN
			OPEN @cursor_syntax;
		ELSE
			OPEN @cursor_syntax;
		END IF;
	END;
	
	--SET @var_ims_schema = IN_imsSchema;
	
	-- capture start time
	SELECT CURRENT TIMESTAMP INTO @info_start_timestamp FROM SYSIBM.SYSDUMMY1;
	

	IF @var_ims_schema IS NULL THEN
		SET @var_ims_schema = '';
	END IF;
	
	-- check IMS Schema name input
	IF TRIM(@var_ims_schema) <> ''  THEN
		-- Capture current schema
		SELECT CURRENT SCHEMA INTO @var_backup_schema FROM SYSIBM.SYSDUMMY1;
		
		-- Set current schema to IMS schema
		SET CURRENT SCHEMA = @var_ims_schema;
	END IF;
	
	-- Check is IMS schema
	SELECT DISTINCT COUNT(TABNAME) INTO @val_temp_int FROM SYSCAT.TABLES 
		WHERE UPPER(TABNAME) LIKE 'IMSLOG%' AND TABSCHEMA = CURRENT SCHEMA;
	
	-- Valid IMS schema will have at least 4 IMS log tables
	IF @val_temp_int <4 THEN
		SET @val_message = 'Error : Invalid ISAMESSO schema - ' || CURRENT SCHEMA;
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;
	
	
	-- ## INPUT VALIDATION
	IF IN_operationCmd = 'ABOUT' THEN
		-- force SQLException to exit with ABOUT info
		SET @val_message = '';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	ELSEIF IN_operationCmd = 'HELP' THEN
		-- force SQLException to exit with HELP
		SET @val_message = '';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
		
	ELSEIF IN_operationCmd = 'SIMULATE' THEN
	 	-- SIMULATE
		-- do nothing
		SET @var_del_mode  = 'calculating';
	ELSEIF IN_operationCmd = 'DELETE' THEN
		-- delete
		-- do nothing
		SET @var_del_mode  = 'deleted';
	ELSE
		-- syntax error , force SQLException to exit
		SET @val_message = 'Error : Invalid delete mode.';
		SELECT INTEGER('A') INTO @val_temp_int FROM SYSIBM.SYSDUMMY1;
	END IF;			
	
		
	-- Calc retain date
	SELECT (CURRENT TIMESTAMP - (IN_retainDays DAY)) INTO @var_tsRetainDate FROM SYSIBM.SYSDUMMY1;
	
	-- Check if IMSLogSystemOps exist
	SELECT DISTINCT COUNT(TABNAME) INTO @var_iLogSystemOpsExist FROM SYSCAT.TABLES 
		WHERE UPPER(TABNAME) = 'IMSLOGSYSTEMOPS' AND TABSCHEMA = CURRENT SCHEMA;
		
	-- IMSLogSystemOpsExist
	IF @var_iLogSystemOpsExist >0 THEN
		IF IN_operationCmd = 'DELETE' THEN
			SET @var_del_tmpCnt = 1;
			WHILE @var_del_tmpCnt>0 DO
				SET @var_strSQL = 'DELETE FROM IMSLogSystemOps WHERE (LOGID, LOGIMSSERVERID) IN ' ||
						          '(SELECT LOGID, LOGIMSSERVERID FROM IMSLogSystemOps WHERE LOGTIME < (CURRENT TIMESTAMP - '|| TRIM(CHAR(IN_retainDays)) ||' DAY) ' ||
						          ' FETCH FIRST 10000 ROWS ONLY )';
				EXECUTE IMMEDIATE @var_strSQL;
				GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
				SET @var_del_LogSystemOps = @var_del_LogSystemOps + @var_del_tmpCnt;
				COMMIT;
			END WHILE;
		ELSE
		
			SET @var_strSQL = 'SELECT COUNT(*) FROM IMSLogSystemOps WHERE LOGTIME < (CURRENT TIMESTAMP - '|| CHAR(IN_retainDays) ||' DAY) FOR READ ONLY';
			PREPARE @stmt_select FROM @var_strSQL;
			OPEN @cursor_select;
			FETCH @cursor_select INTO @var_del_LogSystemOps;
			CLOSE @cursor_select;		
			
		END IF;
	END IF;
	--
	IF IN_operationCmd = 'DELETE' THEN
		SET @var_del_tmpCnt = 1;
		WHILE @var_del_tmpCnt>0 DO
			DELETE FROM IMSLogUserActivity WHERE (LOGID, LOGIMSSERVERID) IN 
				(SELECT LOGID, LOGIMSSERVERID FROM IMSLogUserActivity WHERE LOGTIME < @var_tsRetainDate FETCH FIRST 10000 ROWS ONLY );
			--
			GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
			SET @var_del_LogUserActivity = @var_del_LogUserActivity + @var_del_tmpCnt;
			COMMIT;			
		END WHILE;
	ELSE
		SELECT COUNT(*) INTO @var_del_LogUserActivity FROM IMSLogUserActivity WHERE LOGTIME < @var_tsRetainDate;
	END IF;
	--
	IF IN_operationCmd = 'DELETE' THEN
		SET @var_del_tmpCnt = 1;
		WHILE @var_del_tmpCnt>0 DO
			DELETE FROM IMSLogUserAdminActivity WHERE (LOGID, LOGIMSSERVERID) IN 
				(SELECT LOGID, LOGIMSSERVERID FROM IMSLogUserAdminActivity WHERE LOGTIME < @var_tsRetainDate FETCH FIRST 10000 ROWS ONLY );
			--
			GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
			SET @var_del_LogUserAdminActivity = @var_del_LogUserAdminActivity + @var_del_tmpCnt;
			COMMIT;
		END WHILE;			
	ELSE
		SELECT COUNT(*) INTO @var_del_LogUserAdminActivity FROM IMSLogUserAdminActivity WHERE LOGTIME < @var_tsRetainDate;
	END IF;
	--
	IF IN_operationCmd = 'DELETE' THEN
		SET @var_del_tmpCnt = 1;
		WHILE @var_del_tmpCnt>0 DO
			DELETE FROM IMSLogUserService WHERE (LOGID, LOGIMSSERVERID) IN 
				(SELECT LOGID, LOGIMSSERVERID FROM IMSLogUserService WHERE LOGTIME < @var_tsRetainDate FETCH FIRST 10000 ROWS ONLY );
			--
			GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
			SET @var_del_LogUserService = @var_del_LogUserService + @var_del_tmpCnt;
			COMMIT;
		END WHILE;			
	ELSE
		SELECT COUNT(*) INTO @var_del_LogUserService FROM IMSLogUserService WHERE LOGTIME < @var_tsRetainDate;
	END IF;
	
	
	IF IN_operationCmd = 'DELETE' THEN
		SET @var_del_tmpCnt = 1;
		WHILE @var_del_tmpCnt>0 DO
			DELETE FROM IMSLogSystemMgmtActivity WHERE (LOGID, LOGIMSSERVERID) IN 
				(SELECT LOGID, LOGIMSSERVERID FROM IMSLogSystemMgmtActivity WHERE LOGTIME < @var_tsRetainDate FETCH FIRST 10000 ROWS ONLY );
			--
			GET DIAGNOSTICS @var_del_tmpCnt = ROW_COUNT;
			SET @var_del_LogSystemMgmtActivity = @var_del_LogSystemMgmtActivity + @var_del_tmpCnt;	
			
			--v1.0.1
			COMMIT;
		END WHILE;			
	ELSE
		SELECT COUNT(*) INTO @var_del_LogSystemMgmtActivity FROM IMSLogSystemMgmtActivity WHERE LOGTIME < @var_tsRetainDate;
	END IF;
	
	-- Reset schema to previous default schema
	IF @var_backup_schema IS NOT NULL THEN
		SET CURRENT SCHEMA = @var_backup_schema;
	END IF;
	
	-- capture end time
	SELECT CURRENT TIMESTAMP INTO @info_end_timestamp FROM SYSIBM.SYSDUMMY1;
		
	--calc elapsed time
	SELECT 	
		CASE WHEN DAY(time_diff)>0 THEN TRIM(CHAR(DAY(time_diff))) || 'd ' ELSE '' END
		|| CASE WHEN HOUR(time_diff)>0 THEN TRIM(CHAR(HOUR(time_diff))) || 'h ' ELSE '' END
		|| CASE WHEN MINUTE(time_diff)>0 THEN TRIM(CHAR(MINUTE(time_diff))) || 'm ' ELSE '' END
		|| TRIM(CHAR(SECOND(time_diff))) || 's ' 
		|| SUBSTR(TRIM(CHAR(MICROSECOND(CURRENT TIMESTAMP))),1,4) || 'ms' 
	INTO @info_elapsed_time 
		FROM ( SELECT (@info_end_timestamp - @info_start_timestamp) time_diff FROM SYSIBM.SYSDUMMY1);

	OPEN @cursor_result_output;
	
END
#